import java.util.LinkedList;
import java.util.Random;

/**
 * A test for a tree-based mutex.
 */
public class TreeBasedMutexTest {
	/**
	 * Run a test on the tree-based mutex.
	 * 
	 * @param args	the number of threads to be used
	 */
	public static void main(String[] args) {
		/* Create the threads without the mutex. */
		int numberOfThreads = Integer.parseInt(args[0]);
		LinkedList<Thread> threads = new LinkedList<Thread>();
		for (int i = 0; i < numberOfThreads; i++) {
			threads.add(new MutexTester());
		}
		
		/* Create the tree based mutex with the threads.*/
		TreeBasedMutex treeBasedMutex = new TreeBasedMutex(threads);
		
		/* Initialize the threads with the tree based mutex. */
		for (Thread thread : threads) {
			((MutexTester) thread).setMutex(treeBasedMutex);
		}
		
		/* Start the threads. */
		for (Thread thread : threads) {
			thread.start();
		}
	}
	
	/**
	 * A mutex test thread.
	 */
	public static class MutexTester extends Thread {
		/**
		 * Lock the mutex, print a message, and unlock the mutex.
		 */
		public void run() {
			try {
				while (true) {
					Thread.sleep((new Random()).nextInt(1000));
					mutex.lock();
					System.out.print("Thread " + getId() + " says hi.\n");
					mutex.unlock();
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		/**
		 * Set the mutex.
		 * 
		 * @param mutex	the mutex
		 */
		public void setMutex(TreeBasedMutex mutex) {
			this.mutex = mutex;
		}
		
		/**
		 * The mutex.
		 */
		private TreeBasedMutex mutex;
	}
}


